/*
 * MPDAdmin.java
 *
 * Created on September 30, 2005, 5:57 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package org.bff.javampd;

import java.util.List;
import java.util.ArrayList;
import java.util.Properties;
import org.bff.javampd.events.MPDChangeEvent;
import org.bff.javampd.events.MPDChangeListener;
import org.bff.javampd.exception.MPDAdminException;
import org.bff.javampd.exception.MPDConnectionException;
import org.bff.javampd.exception.MPDResponseException;

/**
 * MPDAdmin represents a administrative controller to a MPD server.  To obtain
 * an instance of the class you must use the <code>getMPDAdmin</code> method from the
 * {@link MPD} connection class.  This class does not have a public constructor
 * (singleton model) so the object must be obtained from the connection object.
 * @author Bill Findeisen
 * @version 1.0
 */
public class MPDAdmin {
    private MPD mpd;
    private Properties prop;
    
    private List<MPDChangeListener> listeners = new ArrayList<MPDChangeListener>();
    
    private static final String MPDPROPKILL = "MPD_ADMIN_KILL";
    private static final String MPDPROPREFRESH = "MPD_ADMIN_REFRESH";
    private static final String OUTPUTPREFIXID = "outputid:";
    private static final String OUTPUTPREFIXNAME = "outputname:";
    private static final String OUTPUTPREFIXENABLED = "outputenabled:";
    
    /**
     * Constructor for MPDAdmin
     * @param mpd the MPD Connection
     */
    MPDAdmin(MPD mpd) {
        this.mpd = mpd;
        this.prop = mpd.getMPDProperties();
    }
    
    /**
     * Adds a {@link MPDChangeListener} to this object to receive
     * {@link MPDChangeEvent}s.
     * @param mcl the MPDChangeListener to add
     */
    public synchronized void addMPDChangeListener(MPDChangeListener mcl) {
        listeners.add(mcl);
    }
    
    /**
     * Removes a {@link MPDChangeListener} from this object.
     * @param mcl the MPDChangeListener to remove
     */
    public synchronized void removePlayerChangedListener(MPDChangeListener mcl) {
        listeners.remove(mcl);
    }
    
    /**
     * Sends the appropriate {@link MPDChangeEvent} to all registered
     * {@link MPDChangeListener}s.
     * @param id the event id to send
     */
    protected synchronized void fireMPDChangeEvent(int id) {
        MPDChangeEvent mce = new MPDChangeEvent(this, id);
        
        for(MPDChangeListener mcl : listeners) {
            mcl.mpdChanged(mce);
        }
    }
    
    /**
     * Kills the mpd connection.
     * @throws org.bff.javampd.exception.MPDConnectionException if there is a problem sending the command
     * @throws org.bff.javampd.exception.MPDAdminException if the MPD response contains an error
     */
    public void killMPD() throws MPDConnectionException, MPDAdminException {
        try {
            mpd.sendMPDCommand(new MPDCommand(prop.getProperty(MPDPROPKILL)));
            fireMPDChangeEvent(MPDChangeEvent.MPD_KILLED);
        } catch(MPDResponseException re) {
            throw new MPDAdminException(re.getMessage(), re.getCommand());
        } catch(Exception e) {
            throw new MPDAdminException(e.getMessage());
        }
    }
    
    /**
     * Updates the MPD database by searching the mp3 directory for new music and
     * removing the old music.
     * @throws org.bff.javampd.exception.MPDConnectionException if there is a problem sending the command
     * @throws org.bff.javampd.exception.MPDAdminException if the MPD response contains an error
     */
    public void updateDatabase() throws MPDConnectionException, MPDAdminException {
        try {
            mpd.sendMPDCommand(new MPDCommand(prop.getProperty(MPDPROPREFRESH)));
            fireMPDChangeEvent(MPDChangeEvent.MPD_REFRESHED);
        } catch(MPDResponseException re) {
            throw new MPDAdminException(re.getMessage(), re.getCommand());
        } catch(Exception e) {
            throw new MPDAdminException(e.getMessage());
        }
    }
    
    /**
     * Returns the daemon uptime in seconds.
     * @return the daemon uptime in seconds
     * @throws org.bff.javampd.exception.MPDAdminException if the MPD response contains an error
     * @throws org.bff.javampd.exception.MPDConnectionException if there is a problem sending the command
     */
    public int getDaemonUpTime() throws MPDConnectionException, MPDAdminException {
        try {
            return(Integer.parseInt(mpd.getServerStat(MPD.StatList.UPTIME)));
        } catch(MPDResponseException re) {
            throw new MPDAdminException(re.getMessage(), re.getCommand());
        } catch(Exception e) {
            throw new MPDAdminException(e.getMessage());
        }
    }
}
